package com.tuya.pulsar.websocket.demo.mq;

import com.tuya.pulsar.websocket.demo.MessageHandler;
import com.tuya.pulsar.websocket.demo.MqConfigs;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.java_websocket.WebSocket;
import org.java_websocket.framing.Framedata;
import org.java_websocket.framing.FramedataImpl1;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import static java.util.concurrent.Executors.newScheduledThreadPool;

public class WssWebSocketConsumer {

    private static Logger logger = LoggerFactory.getLogger(WssWebSocketConsumer.class);

    public static void process() throws URISyntaxException, InterruptedException {
        String accessId = MqConfigs.accessId;
        String accessKey = MqConfigs.accessKey;
        String wssServerUrl = MqConfigs.wssServerUrl;
        if (StringUtils.isAnyBlank(accessId, accessKey, wssServerUrl)) {
            throw new IllegalArgumentException("please check the configuration parameters, accessId,accessKey,wssServerUrl is illegal!");
        }
        final Map<String, String> httpHeaders = getHttpHeaders(accessId, accessKey);
        final String topicUrl = getTopicUrl(wssServerUrl, accessId);
        final ConcurrentHashMap<Integer, WssWebSocketClient> clientPoolMap = new ConcurrentHashMap<>(1);

        WssWebSocketClient wssClient = new WssWebSocketClient(new URI(topicUrl), httpHeaders, new MessageHandler());
        clientPoolMap.put(1, wssClient);
        clientPoolMap.get(1).connectBlocking();
        //
        ScheduledExecutorService schedulePool = newScheduledThreadPool(1);
        schedulePool.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                WssWebSocketClient wssWebSocketClient = clientPoolMap.get(1);
                WebSocket.READYSTATE state = wssWebSocketClient.getReadyState();
                if (state != WebSocket.READYSTATE.OPEN) {
                    if (state == WebSocket.READYSTATE.CLOSED) {
                        try {
                            clientPoolMap.put(1, WssWebSocketClient.reconnect(new URI(topicUrl), httpHeaders, new MessageHandler()));
                            clientPoolMap.get(1).connectBlocking();
                        } catch (InterruptedException | URISyntaxException e) {
                            logger.error("error, ", e);
                        }
                    }
                } else {
                    FramedataImpl1 frameData = new FramedataImpl1(Framedata.Opcode.PING);
                    frameData.setFin(true);
                    wssWebSocketClient.getConnection().sendFrame(frameData);
                    logger.info("webSocket check status is ok.");
                }
            }
        }, 30, 10, TimeUnit.SECONDS);
    }


    private static String getPassword(String accessId, String accessKey) {
        return DigestUtils.md5Hex(accessId + DigestUtils.md5Hex(accessKey)).substring(8, 24);
    }


    private static Map<String, String> getHttpHeaders(String accessId, String accessKey) {
        Map<String, String> map = new HashMap<>(3);
        map.put("username", accessId);
        map.put("password", getPassword(accessId, accessKey));
        return map;
    }


    private static String getTopicUrl(String wssSeverUrl, String accessId) {
        String topicUrl = wssSeverUrl + "ws/v2/consumer/persistent/" + accessId + "/out/" + MqConfigs.topicEnv + "/" + accessId + "-sub" +
                "?ackTimeoutMillis=3000&subscriptionType=Failover";
        return topicUrl;
    }

}
