using System;
using System.Net.WebSockets;
using System.Threading;
using System.Threading.Tasks;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;
using System.Security.Cryptography;
using System.Text;
using System.IO;
using System.Net;

/**
# .net websocket consumer pulsar msg example。
# Dependent package:
    <ItemGroup>
        <None Remove="System.Net.WebSockets" />
        <None Remove="Newtonsoft.Json" />
    </ItemGroup>
    <ItemGroup>
        <PackageReference Include="System.Net.WebSockets" Version="4.3.0" />
        <PackageReference Include="Newtonsoft.Json" Version="13.0.1" />
    </ItemGroup>

# Required parameters

    ACCESS_ID = ""
    ACCESS_KEY = ""
    WSS_SERVER_URL = ""
    MQ_ENV=""

# WSS_SERVER_URL
```
    //CN
    "wss://mqe.tuyacn.com:8285/";
    //US
    "wss://mqe.tuyaus.com:8285/";
    //EU
    "wss://mqe.tuyaeu.com:8285/";
    //IND
    "wss://mqe.tuyain.com:8285/";
```

# Process the messages you receive，business logic is implemented in this method
```
    MessageHandler
```

*/
namespace ws_client
{
    class Program{
        // env
        private static readonly string MQ_ENV_PROD = "event";
        private static readonly string MQ_ENV_TEST = "event-test";

        // accessId, accessKey，serverUrl，MQ_ENV
        private static string ACCESS_ID = "";
        private static string ACCESS_KEY = "";
        private static string WSS_SERVER_URL = "wss://mqe.tuyacn.com:8285/";
        private static string MQ_ENV= MQ_ENV_PROD;


        // basic config
        private static readonly string WEB_SOCKET_QUERY_PARAMS = "?ackTimeoutMillis=3000&subscriptionType=Failover";

        private static readonly TimeSpan PING_INTERVAL_SECONDS = TimeSpan.FromMilliseconds(30000);

        private static readonly int receiveChunkSize = 1024 * 5;

        static void Main(string[] args){
            //params not blank check
            if(string.IsNullOrEmpty(ACCESS_ID)||
               string.IsNullOrEmpty(ACCESS_KEY)||
               string.IsNullOrEmpty(WSS_SERVER_URL)||
               string.IsNullOrEmpty(MQ_ENV)){
                throw new Exception("params is blank, please check!");
            }
            while(!Start().Result){
                Console.WriteLine("------\nstart reconnect...\n------");
                Thread.Sleep(TimeSpan.FromMilliseconds(10000));
            }
        }

        
        public static async Task<Boolean> Start(){
            var topicUrl = GetTopicUrl();
            ClientWebSocket wssClient = GetWebSocket();
            try{
                await wssClient.ConnectAsync(new Uri(topicUrl), CancellationToken.None);
                Console.WriteLine("------\nwssClient connect success.\n------");
                await Task.WhenAll(Receive(wssClient));
                return false;
            }catch(Exception ex){
                //do nothing
                return false;
            }finally{
                wssClient.Dispose();
            }
        }



        public static ClientWebSocket GetWebSocket(){
            System.Net.ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;
            System.Net.ServicePointManager.ServerCertificateValidationCallback = (sender, certificate, chain, sslPolicyErrors) => true;
            
            ClientWebSocket webSocket = null;
            webSocket = new ClientWebSocket();
            webSocket.Options.SetRequestHeader("username", ACCESS_ID);
            webSocket.Options.SetRequestHeader("password", GenPwd());
            webSocket.Options.KeepAliveInterval = PING_INTERVAL_SECONDS;
            return webSocket;
        }

        private static async Task Receive(ClientWebSocket webSocket){
            byte[] buffer = new byte[receiveChunkSize];
            while (webSocket.State == WebSocketState.Open){
                var result = await webSocket.ReceiveAsync(new ArraySegment<byte>(buffer), CancellationToken.None);
                if (result.MessageType == WebSocketMessageType.Close){
                    await webSocket.CloseAsync(WebSocketCloseStatus.NormalClosure, string.Empty, CancellationToken.None);
                }
                var text = System.Text.Encoding.UTF8.GetString(buffer, 0, result.Count);

                JObject jobject = (JObject)JsonConvert.DeserializeObject(text);
                var messageId = jobject["messageId"].ToString();
                string ackStrJson = "{\"messageId\":\""+messageId+"\"}";
                if (messageId != ""){
                    var payload = System.Text.Encoding.UTF8.GetString(Convert.FromBase64String(jobject["payload"].ToString()));
                    //msg handler
                    DateTime s = DateTime.Now;
                    MessageHandler(payload);
                    Console.WriteLine("business processing cost="+(s-DateTime.Now));
                    //send ack
                    await webSocket.SendAsync(new ArraySegment<byte>(System.Text.Encoding.UTF8.GetBytes(ackStrJson)), WebSocketMessageType.Text, true, CancellationToken.None);
                }
            }
        }

        //decrypt_by_aes
        private static string DecryptByAes(string decryptStr, string Key) {
            try{
                byte[] cadenaBytes = Convert.FromBase64String(decryptStr);
                byte[] claveBytes = Encoding.UTF8.GetBytes(Key);

                RijndaelManaged rijndaelManaged = new RijndaelManaged();
                rijndaelManaged.Mode = CipherMode.ECB;
                rijndaelManaged.BlockSize = 128;
                rijndaelManaged.Padding = PaddingMode.Zeros;
                ICryptoTransform desencriptador;
                desencriptador = rijndaelManaged.CreateDecryptor(claveBytes, rijndaelManaged.IV);
                MemoryStream memStream = new MemoryStream(cadenaBytes);
                CryptoStream cryptoStream;
                cryptoStream = new CryptoStream(memStream, desencriptador, CryptoStreamMode.Read);
                StreamReader streamReader = new StreamReader(cryptoStream);
                string resultStr = streamReader.ReadToEnd();

                memStream.Close();
                cryptoStream.Close();
                return resultStr;
            }catch (Exception ex){
                Console.WriteLine(ex);
                return null;
            }
        }


        //message handler
        private static void MessageHandler(string payload){
            try{
                Console.WriteLine("------\npayload="+payload.ToString());
                JObject payloadJson = (JObject)JsonConvert.DeserializeObject(payload);
                Console.WriteLine("payloadJson="+payloadJson.ToString());

                string decryptData = DecryptByAes(payloadJson["data"].ToString(),ACCESS_KEY.Substring(8,16)).Replace("\f","").Replace("\r","").Replace("\n","").Replace("\t","").Replace("\v","").Replace("\b","");
                Console.WriteLine("data="+decryptData);
            }catch(Exception ex){
                Console.WriteLine(ex);
            }
        }


        // pwd
        private static string GenPwd(){
            string md5HexKey = Md5(ACCESS_KEY);
            string mixStr = ACCESS_ID+md5HexKey;
            String md5MixStr = Md5(mixStr);
            return md5MixStr.Substring(8,16);   
        }


        // md5
        private static string Md5(string md5Str){
            MD5CryptoServiceProvider md5 = new MD5CryptoServiceProvider();
            byte[] dataHash = md5.ComputeHash(Encoding.UTF8.GetBytes(md5Str));
            StringBuilder sb = new StringBuilder();
            foreach (byte b in dataHash){
                sb.Append(b.ToString("x2").ToLower());
            }
            return sb.ToString();
        }

        // topicUrl
        private static string GetTopicUrl() {
            string url = WSS_SERVER_URL+"ws/v2/consumer/persistent/"+ACCESS_ID+"/out/"+MQ_ENV+"/"+ACCESS_ID+"-sub"+WEB_SOCKET_QUERY_PARAMS;
            return url;
        }
    }
}
